# -*- coding: utf-8 -*-
import feedparser
from docopt import Dict as DocOptDict
from irc3.plugins.command import command
from irc3.utils import IrcString

from . import Plugin
from ..utils import fmt


class Kernel(Plugin):
    URL = 'https://www.kernel.org/feeds/kdist.xml'

    @command
    def kernel(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Get Linux kernel releases.

        %%kernel
        """
        feed = feedparser.parse(self.URL)
        releases = []
        for e in feed['entries']:
            version, branch = e['title'].split(': ')
            if '(EOL)' in e['description']:
                branch = '%s, {italic}EOL{reset}' % branch
            releases.append('{bold}%s{reset} (%s)' % (version, branch))
        return fmt('[Kernel] {releases}'.format(releases=', '.join(releases)))
