# -*- coding: utf-8 -*-
from docopt import Dict as DocOptDict
from irc3.plugins.command import command
from irc3.utils import IrcString
from irc3 import event

from . import DatabasePlugin
from ..utils import parse_int


class McManiac(DatabasePlugin):
    requires = ['irc3.plugins.command',
                'nxy.plugins.database']

    @command(options_first=True)
    def mcmaniac(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Get a random McManiaC or by index.

        %%mcmaniac [<index>]
        """
        index = args.get('<index>')
        if index:
            index = parse_int(index)
            if not index:
                return
            index, order, _ = index
            order = 'id {order}'.format(order=order)
            extra = 'offset ?'
            binds = [index]
        else:
            order = 'random()'
            extra = ''
            binds = []
        self.cur.execute('''select item,
        (select count(id) from mcmaniacs b where a.id >= b.id) as idx,
        (select count(id) from mcmaniacs) as len
        from mcmaniacs a order by {order} limit 1 {extra}
        '''.format(order=order, extra=extra), binds)
        result = self.cur.fetchone()
        if result:
            return '[{idx}/{len}] {item}'.format(**result)

    @event(r'(?i)^:(?P<mask>\S+) PRIVMSG \S+ :(?P<msg>.*(?P<item>Mc\S+iaC).*)')
    def check(self, mask: str, msg: str, item: str):
        nick = IrcString(mask).nick
        if nick != self.bot.nick and msg != '.reload mcmaniac':
            self.cur.execute('insert into mcmaniacs (item) values (?)', [item])
            self.con.commit()
            self.bot.notice(nick, '[McManiac] added "{}"'.format(item))
