# -*- coding: utf-8 -*-
import irc3
import random

from docopt import Dict as DocOptDict
from irc3.dec import event
from irc3.plugins.command import command
from irc3.utils import IrcString

from . import Plugin
from ..utils import fmt

RAINBOW = ('maroon', 'red', 'orange', 'yellow', 'ltgreen', 'green', 'teal',
           'ltblue', 'blue', 'purple', 'pink')
RAINBOW_LEN = len(RAINBOW)

GNU_LINUX = """I'd Just Like To Interject For A Moment. What you're referring
 to as Linux, is in fact, GNU/Linux, or as I've recently taken to calling it,
 GNU plus Linux. Linux is not an operating system unto itself, but rather
 another free component of a fully functioning GNU system made useful by the
 GNU corelibs, shell utilities and vital system components comprising a full
 OS as defined by POSIX."""


@irc3.plugin
class Useless(Plugin):
    requires = ['irc3.plugins.command']

    @event(r'(?i)^:\S+ PRIVMSG (?P<channel>\S+) :'
           r'.*(?<!gnu[/+])linux(?! kernel).*')
    def linux(self, channel: str):
        if random.randint(0, 4) is 0:
            self.bot.privmsg(channel, GNU_LINUX)

    @event(r'(?i)^:\S+ PRIVMSG (?P<channel>\S+) :(?P<msg>huehuehue)$')
    def huehuehue(self, channel: str, msg: str):
        self.bot.privmsg(channel, msg)

    @event(r'(?i)^:\S+ PRIVMSG (?P<channel>\S+) :same$')
    def same(self, channel: str):
        self.bot.privmsg(channel, 'same')

    @event(r'(?i)^:\S+ PRIVMSG (?P<channel>\S+) :\[(?P<data>.*)\]$')
    def intensifies(self, channel: str, data: str):
        self.bot.privmsg(channel, fmt('{bold}[{data} INTENSIFIES]',
                                      data=data.upper()))

    @command
    def storyofpomfface(self, mask: IrcString, channel: IrcString,
                        args: DocOptDict):
        """Story of pomf face.

        %%storyofpomfface
        """
        for face in (':O C==3', ':OC==3', ':C==3', ':C=3', ':C3', ':3'):
            self.bot.privmsg(channel, face)

    @command
    def choose(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Decides between items separated by comma.

        %%choose <items>...
        """
        choice = random.choice(' '.join(args['<items>']).split(','))
        return '{nick}, I\'d choose: {choice}'.format(nick=mask.nick,
                                                      choice=choice.strip())

    @command
    def jn(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Decides between yes and no (for a question).

        %%jn <question>
        %%jn
        """
        choice = random.choice(['{green}Ja', '{maroon}Nein'])
        return fmt('{nick}: {bold}{color}%s' % choice, nick=mask.nick)

    @command
    def kiss(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Kisses a user.

        %%kiss <nick>
        """
        return fmt('(づ｡◕‿‿◕｡)づ{color}{red}。。・゜゜・。。・゜❤{reset} {nick} '
                   '{color}{red}❤', nick=args['<nick>'])

    @command
    def hug(self, mask: IrcString, channel: IrcString,
            args: DocOptDict) -> str:
        """Hugs a user.

        %%hug <nick>
        """
        return fmt('{color}{red}♥♡❤♡♥{reset} {nick} {color}{red}♥♡❤♡♥',
                   nick=args['<nick>'])

    @command
    def hack(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Hacks (a user).

        %%hack [<nick>]
        """
        nick = args.get('<nick>') or ''
        return 'hacking{}...'.format(' %s' % nick if nick else '')

    @command
    def gay(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Make someone gay (alias to rainbow)

        %%gay <word>...
        """
        return self.rainbow(mask, channel, args)

    @command
    def rainbow(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Colorize a word in rainbow colors.

        %%rainbow <word>...
        """
        last = 0
        word = []
        for i, char in enumerate(' '.join(args.get('<word>'))):
            if char != ' ':
                char = '{color}{%s}%s' % (RAINBOW[last % RAINBOW_LEN], char)
                last += 1
            word.append(char)
        return fmt(''.join(word))

    @command
    def wrainbow(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Colorize words in a sentence with rainbow colors.

        %%wrainbow <words>...
        """
        return fmt(' '.join(['{color}{%s}%s' % (RAINBOW[i % RAINBOW_LEN], word)
                             for i, word in enumerate(args['<words>'])]))
