# -*- coding: utf-8 -*-
import irc3
import requests

from docopt import Dict as DocOptDict
from irc3.plugins.command import command
from irc3.utils import IrcString

from . import Plugin


@irc3.plugin
class Weather(Plugin):
    requires = ['irc3.plugins.command']

    URL = 'https://query.yahooapis.com/v1/public/yql?format=json&q={}'

    def __init__(self, bot: irc3.IrcBot):
        super().__init__(bot)

    @command
    def weather(self, mask: IrcString, channel: IrcString, args: DocOptDict):
        """Saves a message for nick to forward on activity

        %%weather <location>...
        """
        qry = '''select * from weather.forecast where u="c" and woeid in
        (select woeid from geo.places(1) where text="{location}")
        '''.format(location=' '.join(args['<location>']))
        req = requests.get(self.URL.format(qry))
        data = req.json()
        if 'error' in data:
            return '[Weather] Location not found'
        res = data['query']['results']['channel']
        condition = res['item']['condition']
        location = res['location']
        direction = '↑↗→↘↓↙←↖'[round(int(res['wind']['direction']) / 45) % 8]
        return '[Weather] {city}, {region}, {country}: ' \
               '{temp}°{unit_temp} {text}, ' \
               '{direction} {speed} {unit_speed}' \
            .format(city=location['city'],
                    region=location['region'].strip(),
                    country=location['country'],
                    temp=condition['temp'],
                    text=condition['text'],
                    direction=direction,
                    speed=res['wind']['speed'],
                    unit_temp=res['units']['temperature'],
                    unit_speed=res['units']['speed'])
