# -*- coding: utf-8 -*-
import re

from datetime import datetime, timedelta
from pprint import pprint

# @formatter:off
COLORS = dict(
    white=0,    # white
    black=1,    # black
    blue=2,     # blue (navy)
    green=3,    # green
    red=4,      # red
    maroon=5,   # brown (maroon)
    purple=6,   # purple
    orange=7,   # orange (olive)
    yellow=8,   # yellow
    ltgreen=9,  # light green (lime)
    teal=10,    # teal (a green/blue cyan)
    ltcyan=11,  # light cyan (cyan / aqua)
    ltblue=12,  # light blue (royal)
    pink=13,    # pink (light purple / fuchsia)
    grey=14,    # grey
    ltgrey=15,  # light grey (silver)
)

FORMATTING = dict(
    bold='\x02',       # bold
    color='\x03',      # colored text
    italic='\x1D',     # italic text
    underline='\x1F',  # underlined text
    swap='\x16',       # swap bg and fg colors ("reverse video")
    reset='\x0F',      # reset all formatting
    **COLORS
)
# @formatter:on

# Debug helper
pp = pprint


def fmt(__text: str, **kwargs) -> str:
    """Formats a str with `kwargs` and `FORMATTING`."""
    return __text.format(**kwargs, **FORMATTING)


def date_from_iso(date: str) -> datetime:
    return datetime.strptime(date, '%Y-%m-%dT%H:%M:%S.%fZ')


def time_delta(text: str) -> timedelta:
    match = re.match(r'(\d+)(s|m|h|mon|w|y)', text)
    if match:
        num, unit = match.groups()
        num = int(num)
        if unit == 's':
            unit = 'seconds'
        elif unit == 'm':
            unit = 'minutes'
        elif unit == 'h':
            unit = 'hours'
        elif unit == 'd':
            unit = 'days'
        elif unit == 'w':
            unit = 'weeks'
        elif unit == 'mon':
            unit = 'weeks'
            num *= 4
        elif unit == 'y':
            unit = 'weeks'
            num *= 52
        return timedelta(**{unit: num})


def parse_int(val: str) -> tuple:
    try:
        val = int(val)
        if val is not 0:
            if val < 1:
                order, op = 'desc', '<='
                val *= -1
            else:
                order, op = 'asc', '>='
            val -= 1
            return val, order, op
    except ValueError:
        pass
